#include "jwt.h"

#include <chrono>
#include <jwt-cpp/jwt.h>


using namespace std::literals::chrono_literals;

extern "C"
{
    char* create_token(const char* user_id, const char* secret)
    {
        auto token = jwt::create()
                         .set_type("JWS")
                         .set_issuer("KeqingMoe")
                         .set_subject("user_id")
                         .set_audience("web_client")
                         .set_expires_at(std::chrono::system_clock::now() + 3s)
                         .set_payload_claim("user_id", jwt::claim(std::string{user_id}))
                         .sign(jwt::algorithm::hs256{secret});
        return strdup(token.c_str());
    }

    char* get_payload(const char* token)
    {
        auto decoded_token = jwt::decode(token);
        auto payload       = decoded_token.get_payload_claim("user_id").as_string();
        return strdup(payload.c_str());
    }

    int verify_token(const char* token, const char* secret)
    {
        try {
            auto decoded_token = jwt::decode(token);

            auto verifier = jwt::verify()
                                .allow_algorithm(jwt::algorithm::hs256{secret})
                                .with_issuer("KeqingMoe")
                                .with_subject("user_id");
            verifier.verify(decoded_token);
            return 1;
        } catch (...) {
            return 0;
        }
    }
}